#include <string>
#include <stdio.h>

#define APPNAME "SearchMyFriends"
//#define ANDROID_VERSION

//#define INSANE_CAFFE_MOBILE_DEBUGGING 

// Dirty hack (TM)
#include "log.hpp"
#include "log.cpp"
#include "nethelper.cpp"
#include "caffe_mobile.hpp"
#include "caffeclassifier.cpp"


void insane_mobile_debug(char *text)
{
	#ifdef INSANE_CAFFE_MOBILE_DEBUGGING
	debug(text);
	#endif
}

namespace caffe {


void CaffeMobile::init(const char* model_file, const char* trained_file, const char* mean_file, const char* additional_file, const char *wantedCaffeLayerName, bool useExtraLayer, int extraLayerOutputs)
{
	this->wantedCaffeLayerName = string((char*)wantedCaffeLayerName);
	this->weightsFile = string((char*)additional_file);
	this->applySigmoid = true;
    debugSS("Model file:", model_file);
    debugSS("Trained file:", trained_file);
    debugSS("Mean file:", mean_file);
    debugSS("Additional file:", additional_file);    
    debugSS("wanted layer:", wantedCaffeLayerName);    
        
	cc = new CaffeClassifier(model_file, trained_file, mean_file);
	if (cc == NULL)
		error("Failed to create CaffeClassifier instance!");

	caffe_output_layer_size = cc->getLayerChannels("out");
	debugSI("Number of neurons for caffe's output layer:", caffe_output_layer_size);
	caffe_output_layer_data = new float[caffe_output_layer_size];
	if (caffe_output_layer_data == NULL)
		error("failed to allocate memory!");
	
	caffe_wanted_layer_size =  cc->getLayerChannels(wantedCaffeLayerName) * cc->getLayerWidth(wantedCaffeLayerName) * cc->getLayerHeight(wantedCaffeLayerName);
	debugSI("Number of neurons for wanted layer of caffe:", caffe_wanted_layer_size);	
	caffe_wanted_layer_data = new float[caffe_wanted_layer_size];	
	if (caffe_wanted_layer_data == NULL)
		error("failed to allocate memory!");
						
    if (useExtraLayer)
    {
		std::ifstream infile(additional_file);
		if (infile.good())
		{
			nn_loadWeights(additional_file, &additional_layer_weights, &additional_layer_num_inputs, &additional_layer_num_outputs);
			//Value is given so check if it is correct
			if (extraLayerOutputs >= 1)
			{
				if (extraLayerOutputs != additional_layer_num_outputs)
					error("Number of declared output neurons does not fit to number of output neurons in weight file!");
			}
				
			debugSI("Note: as we are using an additional layer the real number of output neurons are now:", additional_layer_num_outputs);
		}
		else
		{
			if (extraLayerOutputs >= 1)
			{
				debug("cannot open weights file. Initialize with random values...");
				additional_layer_num_inputs = 1 + caffe_wanted_layer_size;
				additional_layer_num_outputs = extraLayerOutputs;
				debugSI("Number of inputs for additional layer is set to:", additional_layer_num_inputs);
				debugSI("Number of outputs for additional layer is set to:", additional_layer_num_outputs);					
				additional_layer_weights = nn_allocWeights(additional_layer_num_inputs, additional_layer_num_outputs);	
			}	
			else
			{
				error("cannot initalize weights, no number of output neurons given");
			}	 
		}
		additional_layer_inputs = new float[additional_layer_num_inputs]; //including bias neuron
		if (additional_layer_inputs == NULL)
			error("failed to allocate memory!");
		additional_layer_outputs = new float[additional_layer_num_outputs];
		if (additional_layer_outputs == NULL)
			error("failed to allocate memory!");	

		if ( (caffe_wanted_layer_size+1) != additional_layer_num_inputs)
			error("caffe output layer does not match size of inputs in additional layer!");
		use_additional_layer = true;
	}
	else
	{
		use_additional_layer = false;
	}	
}

CaffeMobile::CaffeMobile(const char* model_file, const char* trained_file, const char* mean_file, const char* additional_file, const char *wantedCaffeLayerName, bool useExtraLayer, int extraLayerOutputs)
{
	init(model_file, trained_file, mean_file, additional_file, wantedCaffeLayerName, useExtraLayer, extraLayerOutputs);
}

CaffeMobile::CaffeMobile(const char* model_file, const char* trained_file, const char* mean_file, const char* additional_file, const char *wantedCaffeLayerName)
{
	if (strlen(additional_file) > 0)
		init(model_file, trained_file, mean_file, additional_file, wantedCaffeLayerName, true, -1);
	else
		init(model_file, trained_file, mean_file, additional_file, wantedCaffeLayerName, false, -1);	
}

CaffeMobile::~CaffeMobile()
{
	if (additional_layer_weights != NULL)
		delete [] additional_layer_weights;
	if (additional_layer_inputs != NULL)
		delete [] additional_layer_inputs;		
	if (additional_layer_outputs != NULL)
		delete [] additional_layer_outputs;
	if (caffe_wanted_layer_data != NULL)
		delete [] caffe_wanted_layer_data;		
	if (caffe_output_layer_data != NULL)
		delete [] caffe_output_layer_data;
	if (cropped_image != NULL)
		delete [] cropped_image;
	if (converted_image != NULL)
		delete [] converted_image;
	delete cc;
}

const float* CaffeMobile::getOutputData()
{
	if (use_additional_layer)
		return additional_layer_outputs;
	else
		return caffe_wanted_layer_data;
}

int CaffeMobile::getOutputSize()
{
	if (use_additional_layer)
		return additional_layer_num_outputs;
	else
		return caffe_wanted_layer_size;
}

const float* CaffeMobile::getCaffeOutputData()
{
		return caffe_wanted_layer_data;
}

int CaffeMobile::getCaffeOutputSize()
{
		return caffe_wanted_layer_size;
}

int CaffeMobile::getPredictedClassification()
{
	return getGreatestIndex(caffe_output_layer_data, caffe_output_layer_size);
}

void CaffeMobile::calcAdditionalLayer()
{
	if (use_additional_layer)	
	{
		nn_fill_input(caffe_wanted_layer_data, additional_layer_inputs, additional_layer_num_inputs); // additional_layer_num_inputs must include the bias neuron
		//normalize(additional_layer_inputs,additional_layer_num_inputs / 2.0f, additional_layer_num_inputs-1);  //Normalize without bias
		nn_forward(additional_layer_inputs, additional_layer_outputs, additional_layer_weights, additional_layer_num_inputs, additional_layer_num_outputs);
		//nn_forward_justmult(additional_layer_inputs, additional_layer_outputs, additional_layer_weights, additional_layer_num_outputs);		
		if (applySigmoid)
			nn_applySigmoid(additional_layer_outputs, additional_layer_num_outputs);
	}
}

void CaffeMobile::predict(unsigned char* image, int size, int imageFormat){
	unsigned char* imageData = convertARGB2BGR(image, size, imageFormat);
	// Calculate output data from image
	cc->predict(imageData);
	addResult(1.0f, true);
	calcAdditionalLayer();
}

void CaffeMobile::predict_cropped(unsigned char* image, int size, int cropped_size, int imageFormat){
	insane_mobile_debug("predict_cropped  started");
	unsigned char* imageData = convertARGB2BGR(image, size, imageFormat);
	
	insane_mobile_debug("predict_cropped after convertARGB2BGR");	
	int offset = (size - cropped_size);
	// Create tmp Array for image data
	if (cropped_image_size != cropped_size)
	{
		if (cropped_image != NULL)
			delete [] cropped_image;
		cropped_image_size = cropped_size;
		cropped_image = new unsigned char[cropped_size * cropped_size * NUM_COLOR_CHANNELS];
		if (cropped_image == NULL)
		{
			error("failed to allocate memory for cropping images");
		}
	}

	// middle	
    copyData(cropped_image, imageData, offset/2, offset/2, size, cropped_size);
	cc->predict(cropped_image);
	addResult(2.0f/6.0f, true);

	// left up
    copyData(cropped_image, imageData, 0, 0, size, cropped_size);
	cc->predict(cropped_image);
	addResult(1.0f/6.0f, false);

	// right up
    	copyData(cropped_image, imageData, offset, 0, size, cropped_size);
	cc->predict(cropped_image);
	addResult(1.0f/6.0f, false);

	// left down
    	copyData(cropped_image, imageData, 0, offset, size, cropped_size);
	cc->predict(cropped_image);
	addResult(1.0f/6.0f, false);

	// right down
    	copyData(cropped_image, imageData, offset, offset, size, cropped_size);
	cc->predict(cropped_image);
	addResult(1.0f/6.0f, false);

	calcAdditionalLayer();
}

CaffeClassifier* CaffeMobile::getClassifier()
{
	return this->cc;
}

void CaffeMobile::copyData(unsigned char* dst,unsigned char* src, int offset_x, int offset_y, int size, int cropped_size)
{
	insane_mobile_debug("copyData begin");
	for(int y=0;y<cropped_size;y++)
	{	
		memcpy(dst + NUM_COLOR_CHANNELS * y * cropped_size,
			src + NUM_COLOR_CHANNELS * (offset_x + (offset_y + y) * size),
			NUM_COLOR_CHANNELS * sizeof(unsigned char) * cropped_size);			
	}
	insane_mobile_debug("copyData end");
		
}

void CaffeMobile::addLayerResult(const char *layerName, float *data, int size, float amplification, bool reset)
{
	insane_mobile_debug("addLayerResult begin");	
	float *out = (float*)cc->getLayerData(layerName);	
	if (reset)
	{		
		for(int i=0;i < size;i++)
			data[i] = out[i] * amplification;	
	}
	else
	{
		for(int i=0;i < size;i++)
			data[i] += out[i] * amplification;		
	}
	insane_mobile_debug("addLayerResult end");	
}

void CaffeMobile::addResult(float amplification, bool reset)
{
	addLayerResult("out",           caffe_output_layer_data, caffe_output_layer_size, amplification, reset);
	addLayerResult(wantedCaffeLayerName.c_str(), caffe_wanted_layer_data, caffe_wanted_layer_size, amplification, reset);	
}

void CaffeMobile::saveAdditionalWeights()
{
	nn_saveWeights(weightsFile.c_str(), additional_layer_weights, additional_layer_num_inputs, additional_layer_num_outputs);
}

void CaffeMobile::resetWeights()
{
	debug("resetting weights to random");
	nn_randomWeights(additional_layer_weights, additional_layer_num_inputs, additional_layer_num_outputs);
}

float *CaffeMobile::getAdditionalWeights()
{
	return additional_layer_weights;
}

int CaffeMobile::getExtraLayerNumInputs()
{
	return additional_layer_num_inputs;
}

int CaffeMobile::getExtraLayerNumOutputs()
{
	return additional_layer_num_outputs;	
}	
	
float* CaffeMobile::getExtraLayerInputs()
{
	return additional_layer_inputs;
}

float* CaffeMobile::getExtraLayerOutputs()
{
	return additional_layer_outputs;	
}	
	
bool CaffeMobile::hasAdditionalLayer()
{
	if (additional_layer_weights != NULL)
		return true;
	else
		return false;
}

void CaffeMobile::setApplySigmoid(bool apply)
{
	applySigmoid = apply;
}	

unsigned char* CaffeMobile::convertARGB2BGR(unsigned char* src, int size, int imageFormat)
{
	unsigned char* dst;
	int *src_argb = (int*)src;
	if (imageFormat == IMAGE_FORMAT_BGR)
	{
		return src;
	}
	else
	{
		if ((converted_size != size) || (converted_image == NULL) )
		{
			if (converted_image != NULL)
				delete [] converted_image;
			converted_image = new unsigned char[size * size * NUM_COLOR_CHANNELS]; //BGR-Format
			converted_size = size;
		}
		dst	= converted_image;
		
		for (int i = 0; i < size * size; i++)
		{
			//ARGB -> BGR
			int col = src_argb[i];
			int b = (col >> 16) & 255;
			int g = (col >>  8) & 255;
			int r = (col >>  0) & 255;
			dst[0] = b;
			dst[1] = g;
			dst[2] = r;
			dst += 3;  
		}
		return converted_image; //not dst, as this got modified!
	}
}
	
} // namespace caffe
