#ifndef CAFFE_MOBILE_HPP_
#define CAFFE_MOBILE_HPP_

#include "caffeclassifier.hpp"

namespace caffe {

class CaffeMobile
{
public:

	CaffeMobile(const char* model_file, const char* trained_file, const char* mean_file, const char* additional_file, const char *wantedCaffeLayerName, bool useExtraLayer, int extraLayerOutputs);
	CaffeMobile(const char* model_file, const char* trained_file, const char* mean_file, const char* additional_file, const char *wantedCaffeLayerName);
	~CaffeMobile();
	const float* getOutputData();
	int getOutputSize();
	const float* getCaffeOutputData();
	int getCaffeOutputSize();	
	int getPredictedClassification();
	void predict(unsigned char* data, int size, int imageFormat);
	void predict_cropped(unsigned char* image, int size, int cropped_size, int imageFormat);
	CaffeClassifier* getClassifier();
	void saveAdditionalWeights();
	void resetWeights();
	bool hasAdditionalLayer();
	float *getAdditionalWeights();
	int getExtraLayerNumInputs();
	int getExtraLayerNumOutputs();
	float* getExtraLayerInputs();
    float* getExtraLayerOutputs();	
	void setApplySigmoid(bool apply);
	static const int IMAGE_FORMAT_BGR = 0;
	static const int IMAGE_FORMAT_ARGB = 1;	
private:
	void init(const char* model_file, const char* trained_file, const char* mean_file, const char* additional_file, const char *wantedCaffeLayerName, bool useExtraLayer, int extraLayerOutputs);

	unsigned char *cropped_image = NULL;
	int cropped_image_size = 0;
	
	int caffe_wanted_layer_size = 0;
	float *caffe_wanted_layer_data = NULL;	
	
	//Could differ to caffe_wanted_layer... but not necessarely
	int caffe_output_layer_size = 0;
	float *caffe_output_layer_data = NULL;	
	
	const int NUM_COLOR_CHANNELS = 3;
	
	bool use_additional_layer = false;
	float *additional_layer_inputs = NULL; //needed because of bias neuron!	
	float *additional_layer_outputs = NULL;
	float *additional_layer_weights = NULL;
	int additional_layer_num_inputs = 0; //Including the bias neuron!
	int additional_layer_num_outputs = 0;
	string wantedCaffeLayerName;
	string weightsFile;
	CaffeClassifier *cc = NULL;
	bool applySigmoid = true;
	
	int converted_size = 0;
	unsigned char *converted_image = NULL;

	void calcAdditionalLayer();
	void copyData(unsigned char* dst,unsigned char* src, int offset_x, int offset_y, int size, int cropped_size);
	void addLayerResult(const char *layerName, float *data, int size, float amplification, bool reset);
	void addResult(float amplification, bool reset);
	unsigned char* convertARGB2BGR(unsigned char* src, int size, int imageFormat);
};

} // namespace caffe

#endif
