#include "log.hpp"
#include "caffeclassifier.hpp"


//#define INSANE_CAFFE_CLASSIFIER_DEBUGGING 


void insane_caffeclassifier_debug(char *text)
{
	#ifdef INSANE_CAFFE_CLASSIFIER_DEBUGGING
	debug(text);
	#endif
}

CaffeClassifier::CaffeClassifier(const char* model_file,
        const char* trained_file,
		   const char* mean_file) {
 
 //Causes memory corruption!

#ifdef CPU_ONLY
  Caffe::set_mode(Caffe::CPU);
#else
  Caffe::set_mode(Caffe::GPU);
#endif

  str_model_file = string(model_file);
  str_trained_file = string(trained_file);
  str_mean_file = string(mean_file);

  debug(str_model_file.c_str());
  debug(str_trained_file.c_str());
  debug(str_mean_file.c_str());
   
  /* Load the network. */
  net_.reset(new Net<float>(str_model_file.c_str(), TEST));
  net_->CopyTrainedLayersFrom(str_trained_file.c_str());

  if (net_->num_inputs() != 1)
   error("Network should have exactly one input.");
   
  if (net_->num_outputs() != 1)
	error("Network should have exactly one output.");

  Blob<float>* input_layer = net_->input_blobs()[0];
  num_channels_ = input_layer->channels();
  
  if(!(num_channels_ == 3))
    error("Input layer should have 3 channels.");
    
  input_geometry_ = cv::Size(input_layer->width(), input_layer->height());

  /* Load the binaryproto mean file. */
  if (str_mean_file != "")
  {
	debugSS("use mean file: ", str_mean_file.c_str());
	setMean(str_mean_file);
	use_mean_file = true;
  }
  else
  {
	debug("Do not use mean file...");  
	use_mean_file = false;	
  }
}


/* Load the mean file in binaryproto format. */
void CaffeClassifier::setMean(const string& mean_file) {
  BlobProto blob_proto;
  ReadProtoFromBinaryFileOrDie(mean_file.c_str(), &blob_proto);

  /* Convert from BlobProto to Blob<float> */
  Blob<float> mean_blob;
  mean_blob.FromProto(blob_proto);
  if (mean_blob.channels() != num_channels_)
    error("Number of channels of mean file doesn't match input layer.");

  /* The format of the mean file is planar 32-bit float BGR or grayscale. */
  std::vector<cv::Mat> channels;
  float* data = mean_blob.mutable_cpu_data();
  for (int i = 0; i < num_channels_; ++i) {
    /* Extract an individual channel. */
    cv::Mat channel(mean_blob.height(), mean_blob.width(), CV_32FC1, data);
    channels.push_back(channel);
    data += mean_blob.height() * mean_blob.width();
  }

  /* Merge the separate channels into a single image. */
  cv::Mat mean;
  cv::merge(channels, mean);

  /* Compute the global mean pixel value and create a mean image
   * filled with this value. */
  cv::Scalar channel_mean = cv::mean(mean);
  mean_ = cv::Mat(input_geometry_, mean.type(), channel_mean);
}

void CaffeClassifier::predict(const unsigned char* image){ //const cv::Mat& img) {

	insane_caffeclassifier_debug("CaffeClassifier::predict started");
	
	Blob<float>* input_layer = net_->input_blobs()[0];
		
	insane_caffeclassifier_debug("CaffeClassifier::predict before Reshape");	
	input_layer->Reshape(1, num_channels_,
	                       input_geometry_.height, input_geometry_.width);
  /* Forward dimension change to all layers. */
  
  	insane_caffeclassifier_debug("CaffeClassifier::predict before Reshape 2");	
  net_->Reshape();

  int width = input_geometry_.width;
  int height = input_geometry_.height; 
  insane_caffeclassifier_debug("CaffeClassifier::predict before create Mat");	
  	
  cv::Mat img = cv::Mat(width, height, CV_8UC3, (void*)image);
  insane_caffeclassifier_debug("CaffeClassifier::predict after create Mat");	

  std::vector<cv::Mat> input_channels;
  insane_caffeclassifier_debug("CaffeClassifier::predict before wrapInputLayer");	  
  wrapInputLayer(&input_channels);

  insane_caffeclassifier_debug("CaffeClassifier::predict after wrapInputLayer");	
  preprocess(img, &input_channels);

  insane_caffeclassifier_debug("CaffeClassifier::predict before ForwardPrefilled");
  net_->ForwardPrefilled();
  insane_caffeclassifier_debug("CaffeClassifier::predict after ForwardPrefilled");  
}

/* Wrap the input layer of the network in separate cv::Mat objects
 * (one per channel). This way we save one memcpy operation and we
 * don't need to rely on cudaMemcpy2D. The last preprocessing
 * operation will write the separate channels directly to the input
 * layer. */
void CaffeClassifier::wrapInputLayer(std::vector<cv::Mat>* input_channels) {
  Blob<float>* input_layer = net_->input_blobs()[0];

  int width = input_layer->width();
  int height = input_layer->height();
  float* input_data = input_layer->mutable_cpu_data();
  for (int i = 0; i < input_layer->channels(); ++i) {
    cv::Mat channel(height, width, CV_32FC1, input_data);
    input_channels->push_back(channel);
    input_data += width * height;
  }
}

void CaffeClassifier::preprocess(const cv::Mat& img,
                            std::vector<cv::Mat>* input_channels) {							
  cv::Mat sample_float;						
  img.convertTo(sample_float, CV_32FC3);
  
  cv::Mat sample_normalized;
  if (use_mean_file)
  {
	cv::subtract(sample_float, mean_, sample_normalized);     
  }
  else
  {
	 sample_normalized = sample_float;
  }


/*
  for (int j =0; j < 3; j++)
  {
	  float *ptr = (float*)input_channels->at(j).data;
	  for (int y = 0; y < img.rows;y++)
	  {	  
		for (int x = 0; x < img.cols;x++)
		{
			cv::Vec3f af = sample_normalized.at<cv::Vec3f>(y, x);
			*ptr = af[j];
			ptr++;
		} 
	  }
	}
*/
  cv::split(sample_normalized, *input_channels);
	
  if(reinterpret_cast<float*>(input_channels->at(0).data)
        != net_->input_blobs()[0]->cpu_data())
    error("Input channels are not wrapping the input layer of the network.");
}


int CaffeClassifier::getLayerWidth(const char *layerName) {
	  string name = string(layerName);
	  if (name == "out")
		  return net_->output_blobs()[0]->width();
	  else
	  	  return net_->blob_by_name(name)->width();
}

int CaffeClassifier::getLayerHeight(const char *layerName) {
	  string name = string(layerName);
	  if (name == "out")
		  return net_->output_blobs()[0]->height();
	  else
		  return net_->blob_by_name(name)->height();
}

int CaffeClassifier::getLayerChannels(const char *layerName) {
	  string name = string(layerName);
	  if (name == "out")
		  return net_->output_blobs()[0]->channels();
	  else
		  return net_->blob_by_name(name)->channels();
}

const float* CaffeClassifier::getLayerData(const char *layerName) {
	  string name = string(layerName);
	  if (name == "out")
		  return net_->output_blobs()[0]->cpu_data();
	  else
		  return (const float*) net_->blob_by_name(name)->cpu_data();	
}


const float* CaffeClassifier::getWeights(int index, int offset) {
		  return (const float*)net_->params()[index]->cpu_data() + net_->params()[index]->offset(offset); //Note: Bias units have a seperate index
}

int CaffeClassifier::getWeightsSize(int index) {
		  return net_->params()[index]->channels() * net_->params()[index]->width() * net_->params()[index]->height() ;
}
