#ifndef CAFFECLASSIFIER_HEADER
#define CAFFECLASSIFIER_HEADER
#include <caffe/caffe.hpp>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>

#include <opencv2/imgproc/imgproc.hpp>
#include <iosfwd>
#include <memory>
#include <string>
#include <utility>
#include <vector>
//#include <glog/logging.h>

using namespace caffe;  // NOLINT(build/namespaces)
using std::string;

class CaffeClassifier {
 public:
	CaffeClassifier(const char* model_file,
	                       const char* trained_file,
						   const char* mean_file);

  void predict(const unsigned char* image);//const cv::Mat& img);

  int getLayerWidth(const char *layerName);
  int getLayerHeight(const char *layerName);
  int getLayerChannels(const char *layerName);
  const float* getLayerData(const char *layerName);
  const float* getWeights(int index, int offset);
  int getWeightsSize(int index); 
  
 private:
  void setMean(const string& mean_file);

  void wrapInputLayer(std::vector<cv::Mat>* input_channels);

  void preprocess(const cv::Mat& img,
                  std::vector<cv::Mat>* input_channels);

 private:
  bool use_mean_file;
  shared_ptr<Net<float> > net_;
  cv::Size input_geometry_;
  int num_channels_;
  cv::Mat mean_;
  std::vector<string> labels_;
  string str_model_file;
  string str_trained_file;
  string str_mean_file;  
};

#endif
