#ifndef COMPARE_CLASSIFIER_HPP_
#define COMPARE_CLASSIFIER_HPP_

class CompareClassifier
{
public: 
	CompareClassifier(int size_single_feature, float learning_rate);
	~CompareClassifier();
	float calc(float *feature1, float *feature2);
	void feedback(float target);	
	void load(std::string filename);
	void save(std::string filename);	
private:

	int sz_all;
	int sz_single;
	float *in;
	float *weights;
	float last_result;
	float lr;
	float *mean;
	int mean_count;
};

CompareClassifier::CompareClassifier(int size_single_feature, float learning_rate)
{
	sz_single = size_single_feature;
	sz_all = sz_single + 1;//2 * sz_single + 1; // +1 for bias
	in = new float[sz_all]();
	weights = new float[sz_all]();
	mean = 	new float[sz_all]();
	nn_randomWeights(weights, sz_all, 1);
	lr = learning_rate;
	mean_count = 0;
}

CompareClassifier::~CompareClassifier()
{
	delete [] weights;
	delete [] in;		
}


float CompareClassifier::calc(float *feature1, float *feature2)
{
	nn_subtract_abs(feature1, feature2, in, sz_single);
	float mc = ( (float) mean_count);
	if (mc < 1.0f)
		mc = 1.0f;
	for (int i = 0; i < sz_all; i++)
	{
		in[i] -= mean[i] / mc;
	}		
	
	//nn_copy(feature1, in, sz_single);
	//nn_copy(feature2, in + sz_single, sz_single);	
	in[sz_all - 1] = 1.0f;

	last_result = sigmoid(dot(in, weights, sz_all));
	return last_result;
}

void CompareClassifier::feedback(float target)
{
	for (int i = 0; i < sz_all; i++)
	{
		mean[i] += in[i];
	}	
	mean_count++;	
	float d = (target - last_result) * lr;
	for (int i = 0; i < sz_all; i++)
	{
		weights[i] += in[i] * d;
	}
}


	
void CompareClassifier::load(std::string filename)
{
	std::ifstream file;
	file.exceptions(std::ios_base::failbit | std::ios_base::eofbit | std::ios_base::badbit);
	file.open(filename, std::ios::out | std::ios::binary);
	file.read((char*)&mean[0], sz_all * sizeof(float));	
	file.read((char*)&weights[0], sz_all * sizeof(float));
	file.close();	
}

void CompareClassifier::save(std::string filename)
{
	std::ofstream file;
	file.exceptions(std::ios_base::failbit | std::ios_base::eofbit | std::ios_base::badbit);
	file.open(filename, std::ios::out | std::ios::binary);
	file.write((char*)&mean[0], sz_all * sizeof(float));	
	file.write((char*)&weights[0], sz_all * sizeof(float));
	file.close();	
}
#endif
