#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <string>
#include <vector>
#include <iostream>
//#include <glog/logging.h>

//#include "caffeclassifier.hpp"

#include "caffe_mobile.cpp" //DIRTY DIRTY DIRTY DIRTY DIRTY DIRTY
#include <sstream>      // std::istringstream
#include <fstream>

#include "compareclassifier.hpp"
#include "svmclassifier.hpp"

using namespace caffe;  // NOLINT(build/namespaces)
using std::string;


//source of SSTR: http://stackoverflow.com/questions/4668760/converting-an-int-to-stdstring
#define SSTR( x ) dynamic_cast< std::ostringstream & >( \
        ( std::ostringstream() << std::dec << x ) ).str()

char *OUTPUT_LAYER = "out";
char *HIDDEN_LAYER_DEFAULT = "fc82";
char *USED_OUTPUT_LAYER = OUTPUT_LAYER;
float LEARNINGFACTOR = 0.0001f;
int FACE_SIZE = 72;
int FACE_CROPPED_SIZE = FACE_SIZE - 8;
int NUM_CAFFELAYER_OUTPUTS = 16;
int NUM_EXTRALAYER_OUTPUTS = 16;
int NUM_EXTRALAYER_INPUTS  = NUM_CAFFELAYER_OUTPUTS + 1;
float *in1;
float *in2;
//float *out1;
//float *out2;
float *weights;
float *grad;

std::string model_file    = "";
std::string trained_file  = "";
std::string mean_file     = "";
std::string sample_file   = "";
std::string sample_path   = "";
std::string weights_file  = "";
std::string compare_classifier_file  = "";
std::string hidden_layer_name = HIDDEN_LAYER_DEFAULT;
int phase_test = -1;
int addition_layer = -1;
int use_hidden_layer = -1;  
int debug_output = 0; 
int classifytest = 0;
int use_l1_norm = 0;
float sameclass_maxerror = 1.0f;
int use_compare_classifier = 0;
int compare_classifier_train = 0;
int use_svm = 0;

struct person {
	std::vector<std::string> *images; 
};

std::vector<person> data; 

void train_net(float *out1, float *out2, bool samePerson)
{
	
	for (int i = 0; i < (NUM_EXTRALAYER_INPUTS) * (NUM_EXTRALAYER_OUTPUTS); i++)
	{
		int x = i / (NUM_EXTRALAYER_INPUTS);
		int y = i % (NUM_EXTRALAYER_INPUTS);

		float sig1 = sigmoid(out1[x]);
		float sig2 = sigmoid(out2[x]);
		float dsig1 = sigmoid_derivate(out1[x]);
		float dsig2 = sigmoid_derivate(out2[x]);

		float delta;

		if (use_l1_norm)
			delta = tanh(sig1 - sig2) * (dsig1*in1[y] - dsig2*in2[y]); 		
		else
			delta = (sig1 - sig2) * (dsig1*in1[y] - dsig2*in2[y]); 
			
		//delta = sig1 * dsig2 * target2[y] + sig2 * dsig1 * target1[y];

		if (!samePerson)
			delta = -delta;

		grad[i] = delta;// + weights[i] * 0.01f; //+(sig1 - 0.5) * in1[y] * 0.5  + (sig2 - 0.5) * in2[y] * 0.5;
	}

	//Don't forget to apply sigmoid, as it is needed later!
	nn_applySigmoid(out1, NUM_EXTRALAYER_OUTPUTS);
	nn_applySigmoid(out2, NUM_EXTRALAYER_OUTPUTS);
	
	
	//float err = dist(out1, out2, NUM_EXTRALAYER_OUTPUTS);
	//if (samePerson || (err <= 1.0f))
		for (int i = 0; i < (NUM_EXTRALAYER_INPUTS) * (NUM_EXTRALAYER_OUTPUTS); i++)
		{	
			weights[i] -= grad[i] * LEARNINGFACTOR;
		}
}

/*
void train_net_easy(float *out1, float *out2, bool samePerson)
{
	
	for (int i = 0; i < (NUM_EXTRALAYER_INPUTS); i++)
	{
		float sig1 = sigmoid(out1[i]);
		float sig2 = sigmoid(out2[i]);
		float dsig1 = sigmoid_derivate(out1[i]);
		float dsig2 = sigmoid_derivate(out2[i]);

		float delta;

		delta = (sig1 - sig2) * (dsig1*in1[i] - dsig2*in2[i]); 

		if (!samePerson)
			delta = -delta;

		grad[i] = delta;
	}

	//Don't forget to apply sigmoid, as it is needed later!
	nn_applySigmoid(out1, NUM_EXTRALAYER_OUTPUTS);
	nn_applySigmoid(out2, NUM_EXTRALAYER_OUTPUTS);
	
	for (int i = 0; i <  (NUM_EXTRALAYER_INPUTS); i++)
	{	
		weights[i] -= grad[i] * LEARNINGFACTOR;
	}
}
*/


std::vector<float> predict(CaffeMobile *cm, std::string filename)
{
  std::vector<float> predictions;
  cv::Mat img = cv::imread(filename, CV_LOAD_IMAGE_COLOR);
  if (!img.empty())
  {
	  if (img.channels() == 3)
	  {
		  if ((img.cols == FACE_SIZE) && (img.rows == FACE_SIZE))
		  {
			  if (img.type() == CV_8UC3)
			  {
				  int sz = cm->getOutputSize();

				  cm->predict_cropped(img.data, FACE_SIZE, FACE_CROPPED_SIZE, CaffeMobile::IMAGE_FORMAT_BGR);  //predict_top_k(imgptr, &sz);//

				  float *ptr = (float*)cm->getOutputData();
				   predictions = std::vector<float>(ptr, ptr + sz);

			  }
			  else
			  {
				 error("image has not type CV_8UC3!");
			  }
	      }
	      else
	      {
				error("image has wrong size!");
		  }
	}
	else
	{
		error("num image channels not equal to 3!");
	}
  }
  else
  {
	  errorSS("unable to load image ", filename.c_str());
  }		
  return predictions;  
}


void readSampleData(std::string training_file, std::string path)
{
	std::cout << "reading samples from " << training_file << std::endl;
	std::cout << "HINT: faces must have successive numbers beginning from 0 (e.g. 0,1,2,3,4...)" << std::endl;
	int idxFace = 0;
	int idxFaceLast = -1;
	std::string face;
	std::ifstream filename(training_file);
	std::string line;
	while (getline(filename, line))
	{
		std::istringstream strm(line);
		strm >> face;
		strm >> idxFace;
		
		if (idxFace >= data.size())
		{
			data.resize(idxFace + 1);
			for (int i = idxFaceLast+1; i <= idxFace; i++)
			{
				person p;
				p.images = new std::vector<string>();
				data[i] = p;
			}
			idxFaceLast = idxFace;
		}
		
		std::string imagefile = path + face;
		std::ifstream infile(imagefile);
		if (infile.good())
			data[idxFace].images->push_back(face);
		else
			errorSS("cannot find file ", imagefile.c_str());
	}
	debug("Checking faces...");		
	for (int i = 0; i < data.size(); i++)
	{
		if (data[i].images->size() <= 2)
			errorSI("face contains less images than two", i);
	}
	debug("Ok");
}

void getTwoSamples(int *idxFace1, int *idxFace2, int *idxSample1, int *idxSample2)
{
  int k = rand() % 1000;
  int numSamples1, numSamples2;
  if (k >= 500)
  {
    *idxFace1 = rand() % (data.size());
    *idxFace2 = *idxFace1;
    numSamples1 = data[*idxFace1].images->size(); 
    
    //std::cout << *idxFace1 << std::endl;
    if (numSamples1 < 2)
		error("sample size is smaller than 2!");      
    *idxSample1 = rand() % (numSamples1);
    *idxSample2 = (*idxSample1 + (rand() % (numSamples1-1))+1 ) % (numSamples1); //another sample        
  } else {
    *idxFace1 = rand() % (data.size());
    *idxFace2 = (*idxFace1 + rand() % (data.size()-1)+1) % (data.size());
    //std::cout << *idxFace1 << std::endl;
       
    numSamples1 = data[*idxFace1].images->size();
    if (numSamples1 < 2)
		error("sample size is smaller than 2!");   
		
    //std::cout << *idxFace2 << std::endl; 		 
    numSamples2 = data[*idxFace2].images->size();  
    if (numSamples2 < 2)
		error("sample size is smaller than 2!");      
    *idxSample1 = rand() % (numSamples1);
    *idxSample2 = rand() % (numSamples2);    
  }
}

float dist_and_log(float *ptrOut1, float *ptrOut2, int sz)
{
	float d;
	
	if (use_l1_norm)
		d = dist_l1(ptrOut1, ptrOut2, sz);
	else
		d = dist_l2(ptrOut1, ptrOut2, sz);
				
	std::cout.precision(4);
	std::cout << std::endl;		
	for (int i = 0; i < sz; i++)
		std::cout << ptrOut1[i] << ", ";
	std::cout << std::endl << std::endl;
	for (int i = 0; i < sz; i++)
		std::cout << ptrOut2[i] << ", ";
	std::cout << std::endl << std::endl << d << std::endl;	
	
	std::cout << std::endl;	
	std::cout << std::endl;	

	return d;
}


int main(int argc, char** argv) {     	
  std::cerr << "USAGE: -deploy deploy.prototxt -model network.caffemodel -samplefile samples.txt -samplepath /home/test/img/"  << std::endl;
  std::cerr << "[-weights weights.dat] [-mean mean.binaryproto] -addlayer:yes/no -usedlayer:out/hidden [-phase:test/train]" << std::endl;
  std::cerr << "[-hiddenlayername fc82] [-inputsize 72] [-cropsize 64] [-additionallayeroutputs 16][-learningrate 0.001][-l1norm][-debug]" << std::endl;
  std::cerr << "[-classifytest 1.0]" << std::endl;
  std::cerr << "[-compareclassifier:train/test][-compareclassifierpath weights][-usesvm]" << std::endl;  
  std::cerr << "!!!!!!!!!!!!DO NOT USE MEAN FILE --> BAD CLASSIFICATION!!!" << std::endl;    
   for (int i = 1; i < argc; ++i)
   {
        string param = string(argv[i]);
        if ((param == "-deploy") && (i < argc-1))
        {
			model_file = string(argv[i+1]);
		}
        if ((param == "-mean") && (i < argc-1))
        {
			mean_file = string(argv[i+1]);
		}
        if ((param == "-model") && (i < argc-1))
        {
			trained_file = string(argv[i+1]);
		}	
        if ((param == "-weights") && (i < argc-1))
        {
			weights_file = string(argv[i+1]);
		}		
        if ((param == "-samplefile") && (i < argc-1))
        {
			sample_file = string(argv[i+1]);
		}	
        if ((param == "-samplepath") && (i < argc-1))
        {
			sample_path = string(argv[i+1]);
		}			
        if ((param == "-hiddenlayername") && (i < argc-1))
        {
			hidden_layer_name = string(argv[i+1]);
		}							
        if ((param == "-inputsize") && (i < argc-1))
        {
			FACE_SIZE = atoi(argv[i+1]);
		}	
        if ((param == "-additionallayeroutputs") && (i < argc-1))
        {
			NUM_EXTRALAYER_OUTPUTS = atoi(argv[i+1]);
		}		
        if ((param == "-cropsize") && (i < argc-1))
        {
			FACE_CROPPED_SIZE = atoi(argv[i+1]);
		}					
        if ((param == "-learningrate") && (i < argc-1))
        {
			LEARNINGFACTOR = atoll(argv[i+1]);
		}						
        if ((param == "-classifytest") && (i < argc-1))
        {
			sameclass_maxerror = atoll(argv[i+1]);
			classifytest = 1;
		}			
	
        if ((param == "-usesvm"))
        {
			use_svm = 1;
		}	
				
        if ((param == "-compareclassifier:test"))
        {
			use_compare_classifier = 1;
			compare_classifier_train = 0;	
		}						
        if ((param == "-compareclassifier:train"))
        {
			use_compare_classifier = 1;
			compare_classifier_train = 1;
			
		}
        if ((param == "-compareclassifierpath") && (i < argc-1))
        {
			compare_classifier_file = string(argv[i+1]);
		}			
		
				
        if ((param == "-l1norm"))
        {
			use_l1_norm = 1;
		}				
        if ((param == "-addlayer:yes"))
        {
			addition_layer = 1;
		}			
        if ((param == "-addlayer:no"))
        {
			addition_layer = 0;
		}			
        if ((param == "-usedlayer:out"))
        {
			use_hidden_layer = 0;
		}			
        if ((param == "-usedlayer:hidden"))
        {
			use_hidden_layer = 1;
		}		
        if ((param == "-phase:test"))
        {
			phase_test = 1;
		}					
        if ((param == "-phase:train"))
        {
			phase_test = 0;
		}
        if ((param == "-debug"))
        {
			debug_output = 1;
		}				
   }
   
   if (model_file == "")
   {
		std::cout << "ENTER PATH TO MODEL (.deploy)" << std::endl;
		std::cin >> model_file;	   
   }   
   
   if (trained_file == "")
   {
		std::cout << "ENTER PATH TO TRAINED FILE (.caffemodel)" << std::endl;
		std::cin >> trained_file;	   
   }   
 
   if (sample_file == "")
   {
		std::cout << "ENTER PATH TO SAMPLE FILE (.txt)" << std::endl;
		std::cin >> sample_file;	   
   }  
   if (sample_path == "")
   {
		std::cout << "ENTER PATH TO SAMPLES (WITH TRAILING /)" << std::endl;
		std::cin >> sample_path;	   
   }         
            
  if (addition_layer == -1)
  {
	std::cout << "USE ADDITIONAL LAYER 1 yes / 0 no" << std::endl;
	std::cin >> addition_layer;
  }

  if (addition_layer)
  {
	if (weights_file == "")
	{
		std::cout << "ENTER PATH TO WEIGHTS FILE (.dat)" << std::endl;
		std::cin >> weights_file;
	}
  }
  
  
	if (use_compare_classifier)
	{
		if (compare_classifier_file == "")
		{
			std::cout << "COMPARE CLASSIFIER PATH:" << std::endl;
			std::cin >> compare_classifier_file;			
		}
	}  
  
  if (use_hidden_layer == -1)
  {
	std::cout << "USE HIDDEN LAYER 1 yes / 0 no" << std::endl;
	std::cin >> use_hidden_layer;
  }  
  if (use_hidden_layer)
	  USED_OUTPUT_LAYER = (char*)hidden_layer_name.c_str();
  else
	  USED_OUTPUT_LAYER = OUTPUT_LAYER;  

  if ((addition_layer) && (phase_test == -1))
  { 
	std::cout << "1 TEST/ 0 TRAIN" << std::endl;
	std::cin >> phase_test;
  } 

  if (sample_path.length() > 0)
  {
	if (sample_path[sample_path.length() - 1] != '/')
		error("sample path must end with a /");
  }
  else
  {
		error("sample path cannot be empty!");
  }			
		
    readSampleData(sample_file, sample_path);
    CaffeMobile *cm = NULL;

	std::cout << "Create Model" << std::endl;
	if (addition_layer)
	{
		cm = new CaffeMobile(model_file.c_str(), trained_file.c_str(), mean_file.c_str(), weights_file.c_str(), USED_OUTPUT_LAYER, true, NUM_EXTRALAYER_OUTPUTS);
		//FOR TRAINING, SIGMOID MUST BE APPLIED SEPERATLY
		cm->setApplySigmoid(false);
		
		if (phase_test == 0)
		{	
			cm->resetWeights();
		}
		else
		{
			cm->setApplySigmoid(true);			
		}
		weights = cm->getAdditionalWeights();
	}
	else
	{
		cm = new CaffeMobile(model_file.c_str(), trained_file.c_str(), mean_file.c_str(), "", USED_OUTPUT_LAYER, false, -1);				
	}		
		
	/*	
	std::ofstream f("weights_internal");
	for (int z=0; z <16; z++)
	{		
		f.write((char*)cm->getClassifier()->getWeights(0,z), cm->getClassifier()->getWeightsSize(0) * sizeof(float));
	}
	f.close();	 
	*/

	if (use_l1_norm)
		std::cout << "Use l1 norm" << std::endl;
	else
		std::cout << "Use l2 norm" << std::endl;				

	NUM_CAFFELAYER_OUTPUTS = cm->getClassifier()->getLayerChannels(USED_OUTPUT_LAYER) * cm->getClassifier()->getLayerWidth(USED_OUTPUT_LAYER) * cm->getClassifier()->getLayerHeight(USED_OUTPUT_LAYER);
		
	std::cout << "Use layer " << USED_OUTPUT_LAYER << " with " << NUM_CAFFELAYER_OUTPUTS << " output neurons" << std::endl;	
	
	if (addition_layer)
	{	
		NUM_EXTRALAYER_INPUTS = cm->getExtraLayerNumInputs();
		NUM_EXTRALAYER_OUTPUTS = cm->getExtraLayerNumOutputs();
		std::cout << "Additional layer has " << NUM_EXTRALAYER_INPUTS << " input and " << NUM_EXTRALAYER_OUTPUTS << " output neurons" << std::endl;	
	 
		grad = nn_allocWeights(NUM_EXTRALAYER_INPUTS, NUM_EXTRALAYER_OUTPUTS);
		in1 = new float[NUM_EXTRALAYER_INPUTS];
		in2 = new float[NUM_EXTRALAYER_INPUTS];
		//out1 = new float[NUM_EXTRALAYER_OUTPUTS];
		//out2 = new float[NUM_EXTRALAYER_OUTPUTS];
	}	
	
	//TODO: Interface would be better approach...
	SVM::SVMClassifier *ccsvm = NULL;
	CompareClassifier *cc = NULL;
	
	if (use_svm)
		ccsvm = new SVM::SVMClassifier(cm->getOutputSize(), LEARNINGFACTOR);
	else
		cc = new CompareClassifier(cm->getOutputSize(), LEARNINGFACTOR);
	
	
	if (use_compare_classifier)
	{
			if (compare_classifier_train == 0)
			{
				if (use_svm)
					ccsvm->load(compare_classifier_file);
				else
					cc->load(compare_classifier_file);				
			}
	}	

	
	std::cout << "Learning rate:\t" <<  LEARNINGFACTOR << std::endl;	
	std::cout << "Face size:\t" <<  FACE_SIZE << std::endl;	
	std::cout << "Face crop size:\t" <<  FACE_CROPPED_SIZE << std::endl;		
	int isSameFace = 0;
	int idxFace1 = 0;
	int idxFace2 = 0;
	int idxSample1 = 0;
	int idxSample2 = 0;  
	std::string face1;
	std::string face2;

	int index_validation_correct = 0;
	int index_validation_count = 0;
	float dist_validation_same_distance = 0.0f;
	int dist_validation_same_count = 0;
	float dist_validation_different_distance = 0.0f;
	int dist_validation_different_count = 0;  
	int counter = 0;
	
	int classify_correct = 0;
	int classify_all = 0;
	
	int compare_correct = 0;
	int compare_all = 0;

  while (true)
  {	  
		getTwoSamples(&idxFace1, &idxFace2, &idxSample1, &idxSample2);
		/*
		idxFace1 = 379;
		idxFace2 = 339;
		idxSample1 = 2;
		idxSample2 = 0;
		*/
		face1 = data[idxFace1].images->at(idxSample1);
		face2 = data[idxFace2].images->at(idxSample2);
		isSameFace = 0;
		if (idxFace1 == idxFace2)
			isSameFace = 1;

		std::vector<float> res1;
		std::vector<float> res2;
				
		if (addition_layer)
		{
			res1 = predict(cm, sample_path + face1);
			nn_copy(cm->getExtraLayerInputs(), in1, NUM_EXTRALAYER_INPUTS);
			res2 = predict(cm, sample_path + face2);	
			nn_copy(cm->getExtraLayerInputs(), in2, NUM_EXTRALAYER_INPUTS);		
			if (phase_test == 0)					
				train_net((float*)&res1[0], (float*)&res2[0], isSameFace);
		}
		else
		{
//			res1 = predict(cm, "/home/benutzer/Downloads/fucking_chris/19.bmp");
//			res2 = predictTest(cm, "/home/benutzer/Downloads/fucking_chris/penis1.bin");

			res1 = predict(cm, sample_path + face1);
			res2 = predict(cm, sample_path + face2);
		}
		
		if (use_compare_classifier)
		{
				float compare_result;
				if (use_svm)	
					compare_result = ccsvm->calc(&res1[0], &res2[0]);
				else
					compare_result = cc->calc(&res1[0], &res2[0]);				
				
				if (compare_classifier_train)
				{
					if (isSameFace)
					{
						if (use_svm)
							ccsvm->feedback(1);
						else
							cc->feedback(1);								
					}
					else
					{
						if (use_svm)
							ccsvm->feedback(0);
						else
							cc->feedback(0);	
					}					
				}
				
				if (compare_result >= 0.5f)
				{
					if (isSameFace)
						compare_correct++;
				}
				else
				{
					if (isSameFace == 0)
						compare_correct++;					
				}
				compare_all++;
				
				if (debug_output)
				{			
						std::cout << "Face " << idxFace1 << ", " << idxFace2 << " was classified as " << compare_result<< std::endl;
				}				
				
		}	
	
			
		int sz_out = res1.size();

		int greatestIdx1 = getGreatestIndex((float*)&res1[0], sz_out);
		int greatestIdx2 = getGreatestIndex((float*)&res2[0], sz_out);	
			
		float err;
		if (use_l1_norm)
			err = dist_l1((float*)&res1[0], (float*)&res2[0], sz_out);  //dist_and_log
		else
			err = dist_l2((float*)&res1[0], (float*)&res2[0], sz_out);  //dist_and_log
					
		if (debug_output)
		{		
			if (!classifytest)
			{
				if ((addition_layer == 0) && (use_hidden_layer == 0))  //Doesn't make always sense!
				{		
					std::cout << "Face " << idxFace1 << " was classified as " << greatestIdx1 << " (1)"<< std::endl;
					std::cout << "Face " << idxFace2 << " was classified as " << greatestIdx2 << " (2)"<< std::endl;
				}
			}
			
			if (isSameFace)
			{
				std::cout << "Distance for same face: \t" << err << std::endl;	
			}
			else
			{
				std::cout << "Distance for different face: \t" << err << std::endl;		
			}
		}

		if (idxFace1 == greatestIdx1)
			index_validation_correct++;
		if (idxFace2 == greatestIdx2)
			index_validation_correct++;		
		index_validation_count += 2;
		
		if (isSameFace)
		{
			dist_validation_same_distance += err;
			dist_validation_same_count += 1;		
		}
		else
		{
			dist_validation_different_distance += err;
			dist_validation_different_count += 1;		
		}
		
		if (classifytest)
		{		
			if (idxFace1 == idxFace2)
			{
				if (!(err > sameclass_maxerror))
					classify_correct++;
			}
			else
			{
				if (err > sameclass_maxerror)
					classify_correct++;		
			}					
			classify_all++;	
		}
			
		if ((counter % 500) == 499)
		{
			float distSame = -1, distDifferent = -1, accurancy = -1;
			
			if (dist_validation_same_count > 0)
				distSame = dist_validation_same_distance / (float)dist_validation_same_count; 
			if (dist_validation_different_count > 0)		
			distDifferent = dist_validation_different_distance / (float)dist_validation_different_count; 		
			std::cout << "Distances Same person: " <<  distSame << " / Different person: " << distDifferent <<  " / ratio: " << distSame / distDifferent << " mean:" << (distSame + distDifferent)/2.0f<<std::endl;	
			if (index_validation_count > 0)
				accurancy = ((float) index_validation_correct) / ((float) index_validation_count);

			if ((addition_layer == 0) && (use_hidden_layer == 0))  //Doesn't make always sense!
				std::cout << "Output Classification accurancy: " <<	accurancy << std::endl;
	
			if (classifytest)
			{	
				float classify_accurancy = ((float)classify_correct) / ((float)classify_all);
				std::cout << "Classification accurancy (based on dist): " << classify_accurancy << std::endl;				
			}		
			
			if (use_compare_classifier)
			{
				float compare_accurancy = ((float)compare_correct)  / ((float) compare_all);
				std::cout << "Classification accurancy (comparison): " << compare_accurancy << std::endl;				
			}				
			index_validation_correct = 0;
			index_validation_count = 0;
			dist_validation_same_distance = 0.0f;
			dist_validation_same_count = 0;
			dist_validation_different_distance = 0.0f;
			dist_validation_different_count = 0;
		}
		
		
		if ((counter % 500) == 499)
		{
			if ( (addition_layer) && (phase_test == 0))
				cm->saveAdditionalWeights();
			
			if (use_compare_classifier)
			{
					if (compare_classifier_train)
					{
						if (use_svm)	
						{
							ccsvm->save(compare_classifier_file);
							ccsvm->load(compare_classifier_file);	
						}	
						else
						{
							cc->save(compare_classifier_file);
							cc->load(compare_classifier_file);								
						}				
					}
			}	
		}
			
	counter++;
  }
}

