#include <netinet/in.h>
#include <fstream>
#include <iostream>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


float sigmoid(float x)
{
	return 1.0f / (exp(-x) + 1.0f);
}

float sigmoid_derivate(float x)
{
	//see:http://stackoverflow.com/questions/7573229/is-this-a-correct-way-to-find-the-derivative-of-the-sigmoid-function-in-python
	float fx = sigmoid(x);
    return fx * (1 - fx);
}


float dot(float *v1, float *v2, int sz)
{
	float res = 0.0f;
	for (int i = 0; i < sz; i++)
	{
		res += v1[i] * v2[i];
	}
	return res;
}


void normalize(float *v1,float factor, int sz)
{
	float res = 0.0f;
	for (int i = 0; i < sz; i++)
	{
		res += v1[i] * v1[i];
	}
	res = sqrtf(res);
	if (res <= 0.000001f)
	{
		res = factor / sqrtf(sz);
		for (int i = 0; i < sz; i++)
		{
			v1[i] = res;
		}		
	}
	else
	{
		res = factor / res;
		for (int i = 0; i < sz; i++)
		{
			v1[i] *= res;
		}
	}
}


float dist_l2(float *v1, float *v2, int sz)
{
	float dist = 0.0f;
	for (int i = 0; i < sz; i++)
		dist += (v1[i] - v2[i]) * (v1[i] - v2[i]);
	return sqrt(dist);			
}

float dist2(float *v1, float *v2, int sz)
{
	float dist = 0.0f;
	for (int i = 0; i < sz; i++)
		dist += (v1[i] - v2[i]) * (v1[i] - v2[i]);
	return dist;			
}

float dist_l1(float *v1, float *v2, int sz)
{
	float dist = 0.0f;
	for (int i = 0; i < sz; i++)
		dist += log(cosh(v1[i] - v2[i]));
	return dist;			
}

int getGreatestIndex(float *in, int sz)
{
	int maxIdx = -1;
	float maxVal = -1000000000.0f;
	for (int i = 0; i < sz; i++)
	{
		if (in[i] > maxVal)
		{
			maxVal = in[i];
			maxIdx = i;
		}
	}
	return maxIdx;
}


void nn_randomWeights(float *ptr, int sz_input, int sz_output)
{
	for (int i = 0; i < sz_input * sz_output; i++)
		ptr[i] = float(rand() % 600) * 0.0001f - 0.03f;
}

float* nn_allocWeights(int sz_input, int sz_output)
{
	float *ptr = new float[sz_input * sz_output];
	nn_randomWeights(ptr, sz_input, sz_output);
	return ptr;
}

void nn_forward(float *input, float* output, float *weights, int sz_input, int sz_output)
{
	for (int i =0; i < sz_output; i++)
	{
		output[i] = dot(input, weights + sz_input*i, sz_input);
	}
}


void nn_forward_justmult(float *input, float* output, float *weights, int sz_input)
{
	for (int i =0; i < sz_input; i++)
	{
		output[i] = weights[i] * input[i];
	}
}

void nn_applySigmoid(float* output, int sz_output)
{
	for (int i =0; i < sz_output; i++)
	{
		output[i] = sigmoid(output[i]);
	}
}

void nn_copy(float* input, float* output, int sz)
{
	for (int i =0; i < sz; i++)
	{
		output[i] = input[i];
	}
}


void nn_subtract(float* input1, float* input2, float* output, int sz)
{
	for (int i =0; i < sz; i++)
	{
		output[i] = input1[i] - input2[i];
	}
}

void nn_subtract_abs(float* input1, float* input2, float* output, int sz)
{
	for (int i =0; i < sz; i++)
	{
		output[i] = abs(input1[i] - input2[i]);
	}
}

void nn_applyNoise(float *dst, int sz)
{
	for (int i = 0; i < sz; i++)
	{
		if ((rand() % 100) > 50)
			if ((rand() % 100) > 50)
				dst[i] = 1.0f;
			else
				dst[i] = 0.0f;	
		else
		{
			//dst[i] *= 2.0f;
		}
	}	
}

void nn_fill_input(float *data_src, float *data_dst, int sz)
{
	//memcpy(data_dst,data_src, (sz-1) * sizeof(float));
	//applyNoise(data_dst, sz);
	//float len = 0.5f;
	for (int i = 0; i < (sz-1); i++)
	{
		data_dst[i] = data_src[i];//sigmoid(data_src[i])*2.0-1.0;
	}	
	data_dst[(sz-1)] = 1.0f;
}

void nn_saveWeights(const char *filename, float *weights, int inputs, int outputs)
{
	uint32_t num_inputs = htonl(inputs);
	uint32_t num_outputs = htonl(outputs);	
	debugSS("save weights", filename);
	//std::cout << "Saveing weights" << std::endl;
	std::ofstream file;
	file.exceptions(std::ios_base::failbit | std::ios_base::eofbit | std::ios_base::badbit);
	file.open(filename, std::ios::out | std::ios::binary);
	file.write((char*)&num_inputs, sizeof(uint32_t));
	file.write((char*)&num_outputs, sizeof(uint32_t));	
	file.write((char*)&weights[0], inputs * outputs * sizeof(float));
	file.close();	
}

void nn_loadWeights(const char *filename, float **weights, int *inputs, int *outputs)
{
	uint32_t num_inputs;
	uint32_t num_outputs;		
	//std::cout << "Loading weights" << std::endl;
	debugSS("load weights", filename);	
	std::ifstream file;
	file.exceptions(std::ios_base::failbit | std::ios_base::eofbit | std::ios_base::badbit);
	file.open(filename, std::ios::in | std::ios::binary);
	file.read((char*)&num_inputs, sizeof(uint32_t));
	file.read((char*)&num_outputs, sizeof(uint32_t));
	
	num_inputs = ntohl(num_inputs);
	num_outputs = ntohl(num_outputs);
	*inputs = (int)num_inputs;
	*outputs = (int)num_outputs;	
	debugSI("read inputs ", num_inputs);
	debugSI("read outpus ", num_outputs);		
	
	*weights = nn_allocWeights(num_inputs, num_outputs);
		
	file.read((char*)*weights, num_inputs * num_outputs * sizeof(float));
	file.close();	
}	
