#ifndef SVM_CLASSIFIER_HPP_
#define SVM_CLASSIFIER_HPP_


namespace SVM
{
	#include "svm.h"
	#include "svm.cpp"

	class SVMClassifier
	{
	public: 
		SVMClassifier(int size_single_feature, float learning_rate);
		~SVMClassifier();
		float calc(float *feature1, float *feature2);
		void feedback(int target);	
		void load(std::string filename);
		void save(std::string filename);	
	private:
		struct svm_model* model;
		std::stringstream strm;
		int sz_single;
		int sz_all;	
		float *in;
		float std;
	};

	SVMClassifier::SVMClassifier(int size_single_feature, float std)
	{
		this->std = std;
		sz_single = size_single_feature;
		in = new float[sz_single]();
		model = NULL;
	}

	SVMClassifier::~SVMClassifier()
	{
		delete [] in;		
	}


	float SVMClassifier::calc(float *feature1, float *feature2)
	{
		for (int i = 0; i < sz_single; i++)
		{
			in[i] =  (feature1[i] - feature2[i]);  //abs because of symetry (swapping feature1 and feature2 should be the same value)
			if (in[i] < 0.0f)
				in[i] = -in[i];
		}
		//normalize(in, sz_single, sz_single);
		
		if (model != NULL)
		{
			struct svm_node x[sz_single + 1];

			x[sz_single].index = -1;
			x[sz_single].value = 0.0;
			for (int i = 0; i < sz_single; i++)
			{
				x[i].index = i+1;

				x[i].value = in[i];
			}

			return (float)svm_predict(model, x);
		}
		else
		{
			return 0.0f;
		}
	}

	void SVMClassifier::feedback(int target)
	{
		strm << target << " ";
		for (int i = 0; i < sz_single; i++)
			strm << (i+1) << ":"  << in[i] << " ";
		strm << std::endl;
	}
		
	void SVMClassifier::load(std::string filename)
	{
		model=svm_load_model(filename.c_str()); //.model file
	}

	void SVMClassifier::save(std::string filename)
	{
		std::ofstream file;
		file.exceptions(std::ios_base::failbit | std::ios_base::eofbit | std::ios_base::badbit);
		file.open(filename + ".data", std::ios::out | std::ios::binary);
		file << strm.str();
		file.close();	
		std::stringstream cmd;
		//svm-train.exe -g 0.00001 my5 my5.model //-c 1024 -g 0.0000125
		cmd << "./svm-train -g 0.0001 -c 100 -t 2 \"" << filename << ".data\" \"" << filename << "\"";
		std::cout << cmd.str() << std::endl;
		system(cmd.str().c_str());
	}
}
#endif
