package openCvFaces.utils;

import java.util.ArrayList;
import java.util.Collection;

import org.opencv.core.Core;
import org.opencv.core.Mat;
import org.opencv.core.MatOfRect;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;
import org.opencv.highgui.Highgui;
import org.opencv.imgproc.Imgproc;
import org.opencv.objdetect.CascadeClassifier;

public class SearchFaces implements SearchFace
{
	private ArrayList<Mat> faces = new ArrayList<Mat>();
	int maxScoreVal = -1;
	Mat maxScoreMat = null;
	private Mat tmp,tmp2;
	private CascadeClassifier faceDetector;
	private MatOfRect faceDetections;
	
	public SearchFaces()
	{
		faceDetector = new CascadeClassifier();
		faceDetector.load("./resources/haarcascade_frontalface_alt.xml");
		faceDetections = new MatOfRect();
		System.loadLibrary(Core.NATIVE_LIBRARY_NAME);
	}

	public ArrayList<Mat> getFace(Mat image)
	{
		try
		{		
			faces.clear();
			maxScoreVal = -1000000000;
			maxScoreMat = null;
		
		if(image == null)
		{
			System.out.println("Image was null");
			return null;
		}
		tmp = image.clone();
	    
		faceDetector.detectMultiScale(tmp, faceDetections);
	    System.out.println(String.format("Detected %s faces", faceDetections.toArray().length));
	 
	    for (Rect rect : faceDetections.toArray()) {
	  
	        int x = rect.x;
	        int y = rect.y;
	        int width = rect.width;
	        int height = rect.height;
	        
	        x = x - width/8;
	        y = y - height/8;
	        width = width + width/4;
	        height = height + height/4;
	        if (x < 0) x = 0;    
	        if (y < 0) y = 0;
	        
	        if (width > image.cols()) width = image.cols();
	        if (height > image.rows()) height = image.rows();	           
	        if (x+width > image.cols()) x = image.cols() - width;
	        if (y+height > image.rows()) y = image.rows() - height;	 
	        if (width > height) width = height;
	        if (height > width) height = width;          
	            
	        rect.x = x;
	        rect.y = y;
	        rect.width = width;
	        rect.height = height;
	        
	    	tmp2 = new Mat(tmp,rect);
	    	tmp.release();
	    	//Imgproc.cvtColor(tmp2,tmp, Imgproc.COLOR_RGB2GRAY);
	    	//tmp2.release();
	    	
	        int dx = (rect.x + rect.width/2) - image.cols() / 2;
	        int dy = (rect.y + rect.height/2) - image.rows() / 2;	
	        if (dx < 0) dx = -dx;
	        if (dy < 0) dy = -dy;        
	        
	        int score = -2 * dx / 8 - dy / 8 + rect.width + rect.height;
	        if (score > maxScoreVal)
	        {
				if (maxScoreMat != null)
					faces.add(maxScoreMat);				
				maxScoreVal = score;
				maxScoreMat = tmp2;
			}
			else
			{	    	
				faces.add(tmp2);
	    	}
	    		
	    	tmp = image.clone();
	    }
	    tmp.release();
	    //tmp2.release();
	    
	    if (maxScoreMat != null)
			faces.add(maxScoreMat);
	    
	    return faces;
		}catch(Exception e)
		{
			 tmp.release();
			 tmp2.release();
			 return null;
		}
	}

}
